# -*- coding: utf-8 -*-

import boto3
import botocore.exceptions


class s3_server:
    def __init__(self, bucket, region):
        self.bucket = bucket
        self.region = region
        self.client = boto3.client("s3", region_name=region)

    def get_files_and_folders(self, prefix="", continuation_token="", max_keys=1000):
        """
            Get the files and folders that exist at a given prefix. This will
            retrieve max_keys files (default 1000). If there are more files
            available, next_continuation_token will be returned. If it's
            provided as continuation_token, the next set of files will fetched.

            Returns a tuple that contains a list of files, a list of folders,
            and next_continuation_token.
        """
        kwargs = {
            "Bucket": self.bucket,
            "Prefix": prefix,
            "Delimiter": "/",
            "MaxKeys": max_keys,
        }
        if continuation_token:
            # Only set if continuation_token is set
            kwargs["ContinuationToken"] = continuation_token
        page = self.client.list_objects_v2(**kwargs)

        files = []
        folders = []
        next_continuation_token = ""
        for key_prefix in page.get("CommonPrefixes", []):
            if prefix in key_prefix.get("Prefix"):
                folders.append(key_prefix.get("Prefix"))
        for s3_object in page.get("Contents", []):
            if s3_object["Size"] == 0:
                continue
            files.append(s3_object)
        if page.get("IsTruncated"):
            next_continuation_token = page.get("NextContinuationToken", "")
        return (files, folders, next_continuation_token)

    def key_exists(self, key):
        """ Try to fetch the metadata for an object. If the object does not
            exist, head_object will raise an exception. Returns True if the
            object exists
        """
        try:
            head = self.client.head_object(Bucket=self.bucket, Key=key)
            if head.get("ContentLength") == 0:
                # Disregard empty objects
                return False
        except (
            botocore.exceptions.ParamValidationError,
            botocore.exceptions.ClientError,
        ):
            return False
        return True

    def create_signed_url(self, key):
        return self.client.generate_presigned_url(
            "get_object", Params={"Bucket": self.bucket, "Key": key}, ExpiresIn=90
        )



# -*- coding: utf-8 -*-

from flask import (
    Flask,
    redirect,
    render_template,
    request,
    jsonify,
    make_response,
    send_from_directory,
)
from flask_bootstrap import Bootstrap
from tuxpub.filters import datetimeformat, file_name, file_size, folder_name
from tuxpub.resources import s3_server
from tuxpub.config import TuxConfig


def create_app():
    app = Flask(__name__, instance_relative_config=True, static_folder="static")
    Bootstrap(app)
    app.jinja_env.filters["datetimeformat"] = datetimeformat
    app.jinja_env.filters["file_name"] = file_name
    app.jinja_env.filters["file_size"] = file_size
    app.jinja_env.filters["folder_name"] = folder_name

    config = TuxConfig()
    s3_handler = s3_server(bucket=config.S3_BUCKET, region=config.S3_REGION)

    @app.context_processor
    def add_site_name():
        return {"SITE_TITLE": config.SITE_TITLE}

    @app.route("/robots.txt")
    def static_folder():
        return send_from_directory(app.static_folder, request.path[1:]), 200

    @app.route("/favicon.ico")
    def favicon():
        return render_template("404.html"), 404

    @app.route("/", defaults={"dummy": ""})
    @app.route("/<path:dummy>")
    def fallback(dummy=""):
        """ Process an arbitrary request. """
        url = dummy

        # If the url is to a specific object, serve it.
        if s3_handler.key_exists(url):
            return redirect(s3_handler.create_signed_url(url), code=302)

        # Add a trailing slash to turn the url into a directory, unless we're
        # serving the index page (in which case url will be empty, and adding a
        # / will break things)
        if not url.endswith("/") and url != "":
            destination = request.base_url + "/"
            if request.args:
                destination += "?"
                for key, value in request.args.items():
                    destination += f"{key}={value}&"
            return redirect(destination, 302)

        # If the request is for the index page and ROOT_INDEX_LISTING is disabled,
        # return root.html template
        if not config.ROOT_INDEX_LISTING and (url.count("/") < 1 or url in ["", "/"]):
            # Short circuit requests for the index page when ROOT_INDEX_LISTING
            # is disabled
            return render_template("root.html"), 200

        # Retrieve the files and folders in the local path
        (files, folders, next_continuation_token) = s3_handler.get_files_and_folders(
            url
        )

        # No files or folders found.
        if len(files) == 0 and len(folders) == 0:
            return render_template("404.html"), 404

        # Return file listing in the requested format (html by default)
        if request.args.get("export"):
            return export_files(
                export_format=request.args.get("export"),
                files=files,
                folders=folders,
                base_url=request.base_url,
            )
        else:
            return render_template(
                "files.html",
                files=files,
                folders=folders,
                base_url=request.base_url,
                show_parent=(url != ""),
            )

    def export_files(export_format, files=[], folders=[], base_url=""):
        """ Return the list of files in an alternate format (like json) """

        if export_format not in ["json"]:
            return "Export format not supported", 501

        if export_format == "json":
            json = {"files": [], "folders": []}
            for f in files:
                json["files"].append(
                    {
                        "Url": base_url + f["Key"],
                        "Size": f["Size"],
                        "LastModified": f["LastModified"],
                        "ETag": f["ETag"],
                    }
                )
            for f in folders:
                json["folders"].append(
                    {"Url": base_url + f,}
                )
            return make_response(jsonify(json), 200)

    return app



# -*- coding: utf-8 -*-

import os
from distutils.util import strtobool


class TuxConfig:
    def __init__(self):
        self.config_strings = {}
        self.config_strings["S3_BUCKET"] = None
        self.config_strings["S3_REGION"] = None
        self.config_strings["SITE_TITLE"] = "Tuxpub"

        self.config_bools = {}
        self.config_bools["ROOT_INDEX_LISTING"] = True
        for config_var in self.config_bools:
            if config_var in os.environ:
                self.config_bools[config_var] = bool(strtobool(os.environ[config_var]))

        for config_var in self.config_strings:
            if config_var in os.environ:
                self.config_strings[config_var] = os.environ[config_var]

        assert (
            self.config_strings["S3_BUCKET"] is not None
        ), "Required env var S3_BUCKET not set"
        assert (
            self.config_strings["S3_REGION"] is not None
        ), "Required env var S3_REGION not set"

    @property
    def S3_BUCKET(self):
        return self.config_strings["S3_BUCKET"]

    @property
    def S3_REGION(self):
        return self.config_strings["S3_REGION"]

    @property
    def SITE_TITLE(self):
        return self.config_strings["SITE_TITLE"]

    @property
    def ROOT_INDEX_LISTING(self):
        return self.config_bools["ROOT_INDEX_LISTING"]
